@@ -162,8 +162,6 @@ init(struct ip6t_entry_match *m, unsigned int *nfcache)
 	info->matchflags = 0x00;
 	info->invflags = 0x00;
 	info->modeflag = 0x00;
-	/* No caching (yet) */
-	*nfcache |= NFC_UNKNOWN;
 }
 
 static unsigned int
@@ -298,19 +296,18 @@ save(const struct ip6t_ip6 *ip,
 }
 
 static
-struct ip6tables_match ipv6header
-= { NULL,
-    "ipv6header",
-    IPTABLES_VERSION,
-    IP6T_ALIGN(sizeof(struct ip6t_ipv6header_info)),
-    IP6T_ALIGN(sizeof(struct ip6t_ipv6header_info)),
-    &help,
-    &init,
-    &parse,
-    &final_check,
-    &print,
-    &save,
-    opts
+struct ip6tables_match ipv6header = {
+	.name		= "ipv6header",
+	.version	= IPTABLES_VERSION,
+	.size		= IP6T_ALIGN(sizeof(struct ip6t_ipv6header_info)),
+	.userspacesize	= IP6T_ALIGN(sizeof(struct ip6t_ipv6header_info)),
+	.help		= &help,
+	.init		= &init,
+	.parse		= &parse,
+	.final_check	= &final_check,
+	.print		= &print,
+	.save		= &save,
+	.extra_opts	= opts,
 };
 
 void _init(void)
