@@ -8,6 +8,8 @@
  *
  * Author: James Morris <jmorris@intercode.com.au>
  *
+ * 07-11-2001 Modified by Fernando Anton to add support for IPv6.
+ *
  * Copyright (c) 2000-2001 Netfilter Core Team
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +28,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/time.h>
+#include <sys/types.h>
 
 #include <libipq/libipq.h>
 
@@ -50,9 +54,11 @@ enum {
 	IPQ_ERR_NLRECV,
 	IPQ_ERR_SEND,
 	IPQ_ERR_SUPP,
-	IPQ_ERR_RECVBUF
+	IPQ_ERR_RECVBUF,
+	IPQ_ERR_TIMEOUT,
+        IPQ_ERR_PROTOCOL
 };
-#define IPQ_MAXERR IPQ_ERR_RECVBUF
+#define IPQ_MAXERR IPQ_ERR_PROTOCOL
 
 struct ipq_errmap_t {
 	int errcode;
@@ -72,7 +78,9 @@ struct ipq_errmap_t {
 	{ IPQ_ERR_NLRECV, "Received error from netlink" },
 	{ IPQ_ERR_SEND, "Failed to send netlink message" },
 	{ IPQ_ERR_SUPP, "Operation not supported" },
-	{ IPQ_ERR_RECVBUF, "Receive buffer size invalid" }
+	{ IPQ_ERR_RECVBUF, "Receive buffer size invalid" },
+	{ IPQ_ERR_TIMEOUT, "Timeout"},
+	{ IPQ_ERR_PROTOCOL, "Invalid protocol specified" }
 };
 
 static int ipq_errno = IPQ_ERR_NONE;
@@ -81,7 +89,8 @@ static ssize_t ipq_netlink_sendto(const struct ipq_handle *h,
                                   const void *msg, size_t len);
 
 static ssize_t ipq_netlink_recvfrom(const struct ipq_handle *h,
-                                    unsigned char *buf, size_t len);
+                                    unsigned char *buf, size_t len,
+                                    int timeout);
 
 static ssize_t ipq_netlink_sendmsg(const struct ipq_handle *h,
                                    const struct msghdr *msg,
@@ -110,9 +119,11 @@ static ssize_t ipq_netlink_sendmsg(const struct ipq_handle *h,
 }
 
 static ssize_t ipq_netlink_recvfrom(const struct ipq_handle *h,
-                                    unsigned char *buf, size_t len)
+                                    unsigned char *buf, size_t len,
+                                    int timeout)
 {
-	int addrlen, status;
+	unsigned int addrlen;
+	int status;
 	struct nlmsghdr *nlh;
 
 	if (len < sizeof(struct nlmsgerr)) {
@@ -120,6 +131,37 @@ static ssize_t ipq_netlink_recvfrom(const struct ipq_handle *h,
 		return -1;
 	}
 	addrlen = sizeof(h->peer);
+
+	if (timeout != 0) {
+		int ret;
+		struct timeval tv;
+		fd_set read_fds;
+		
+		if (timeout < 0) {
+			/* non-block non-timeout */
+			tv.tv_sec = 0;
+			tv.tv_usec = 0;
+		} else {
+			tv.tv_sec = timeout / 1000000;
+			tv.tv_usec = timeout % 1000000;
+		}
+
+		FD_ZERO(&read_fds);
+		FD_SET(h->fd, &read_fds);
+		ret = select(h->fd+1, &read_fds, NULL, NULL, &tv);
+		if (ret < 0) {
+			if (errno == EINTR) {
+				return 0;
+			} else {
+				ipq_errno = IPQ_ERR_RECV;
+				return -1;
+			}
+		}
+		if (!FD_ISSET(h->fd, &read_fds)) {
+			ipq_errno = IPQ_ERR_TIMEOUT;
+			return 0;
+		}
+	}
 	status = recvfrom(h->fd, buf, len, 0,
 	                      (struct sockaddr *)&h->peer, &addrlen);
 	if (status < 0) {
@@ -130,6 +172,10 @@ static ssize_t ipq_netlink_recvfrom(const struct ipq_handle *h,
 		ipq_errno = IPQ_ERR_RECV;
 		return -1;
 	}
+	if (h->peer.nl_pid != 0) {
+		ipq_errno = IPQ_ERR_RECV;
+		return -1;
+	}
 	if (status == 0) {
 		ipq_errno = IPQ_ERR_NLEOF;
 		return -1;
@@ -157,9 +203,8 @@ static char *ipq_strerror(int errcode)
 
 /*
  * Create and initialise an ipq handle.
- * FIXME: implement flags.
  */
-struct ipq_handle *ipq_create_handle(u_int32_t flags)
+struct ipq_handle *ipq_create_handle(u_int32_t flags, u_int32_t protocol)
 {
 	int status;
 	struct ipq_handle *h;
@@ -169,8 +214,19 @@ struct ipq_handle *ipq_create_handle(u_int32_t flags)
 		ipq_errno = IPQ_ERR_HANDLE;
 		return NULL;
 	}
+	
 	memset(h, 0, sizeof(struct ipq_handle));
-	h->fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_FIREWALL);
+	
+        if (protocol == PF_INET)
+                h->fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_FIREWALL);
+        else if (protocol == PF_INET6)
+                h->fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_IP6_FW);
+        else {
+		ipq_errno = IPQ_ERR_PROTOCOL;
+		free(h);
+		return NULL;
+        }
+        
 	if (h->fd == -1) {
 		ipq_errno = IPQ_ERR_SOCKET;
 		close(h->fd);
@@ -226,11 +282,14 @@ int ipq_set_mode(const struct ipq_handle *h,
 	return ipq_netlink_sendto(h, (void *)&req, req.nlh.nlmsg_len);
 }
 
-/* Note: timeout is not yet implemented */
+/*
+ * timeout is in microseconds (1 second is 1000000 (1 million) microseconds)
+ *
+ */
 ssize_t ipq_read(const struct ipq_handle *h,
                  unsigned char *buf, size_t len, int timeout)
 {
-	return ipq_netlink_recvfrom(h, buf, len);
+	return ipq_netlink_recvfrom(h, buf, len, timeout);
 }
 
 int ipq_message_type(const unsigned char *buf)
